#include <iostream>
#include <cstring>
using namespace std;

#include <tcl.h>
#include <tk.h>

extern "C" {
  int SAOAppInit(Tcl_Interp *interp);
  int SAOLocalMainHook(int* argc, char*** argv);

  void TclSetStartupScriptFileName(const char*);

/*  int Zvfs_Init(Tcl_Interp*);
  int Zvfs_Mount(Tcl_Interp*, char*, char *);*/

  int Blt_Init(Tcl_Interp*);
  int Tktable_Init(Tcl_Interp*);
  int Checkdns_Init(Tcl_Interp*);
  int Saotk_Init(Tcl_Interp*);
  int Tkhtml_Init(Tcl_Interp*);
  int Tkmpeg_Init(Tcl_Interp*);

  int Tkimg_Init(Tcl_Interp*);
  int Zlibtcl_Init(Tcl_Interp*);
  int Jpegtcl_Init(Tcl_Interp*);
  int Tkimgjpeg_Init(Tcl_Interp*);
  int Tifftcl_Init(Tcl_Interp*);
  int Tkimgtiff_Init(Tcl_Interp*);
  int Pngtcl_Init(Tcl_Interp*);
  int Tkimgpng_Init(Tcl_Interp*);
  int Tkimggif_Init(Tcl_Interp*);
  int Tkimgppm_Init(Tcl_Interp*);
  int Tkimgbmp_Init(Tcl_Interp*);
  int Tkimgxbm_Init(Tcl_Interp*);
  int Tkimgwindow_Init(Tcl_Interp*);

  int Tclxpa_Init(Tcl_Interp*);
  int Iis_Init(Tcl_Interp*);

#ifdef _MACOSX
  int Tkmacosx_Init(Tcl_Interp*);
#endif
}

static char* appname = NULL;
static int console =0;
char* dupstr(const char* str);

int SAOLocalMainHook(int* argcPtr, char*** argvPtr)
{
  // sync C++ io calls with C io calls
  ios::sync_with_stdio();

  // save real application name
  int argc = *argcPtr;
  char** argv = *argvPtr;
  appname = dupstr(argv[0]);

  // set default encoding dir
//  Tcl_SetDefaultEncodingDir("./zvfsmntpt/tcl8.4");

  // so that tcl and tk know where to find their libs
  // we do it here before InitLibraryPath is called
//  putenv("TCL_LIBRARY=./zvfsmntpt/tcl8.4");
//  putenv("TK_LIBRARY=./zvfsmntpt/tk8.4");

  // trap for -console, else set our startup script
  if (argc<2 || strcmp(argv[1],"-console"))
    TclSetStartupScriptFileName("/usr/share/ds9/src/ds9.tcl");
  else
    console =1;
}

int SAOAppInit(Tcl_Interp *interp)
{
  // reset argv0
  if (appname) {
    Tcl_SetVar(interp, "argv0", appname, TCL_GLOBAL_ONLY);
    delete appname;
  }

  // We have to initialize the virtual filesystem before calling
  // Tcl_Init().  Otherwise, Tcl_Init() will not be able to find
  // its startup script files.
/*  if (Zvfs_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "zvfs", Zvfs_Init, 
		     (Tcl_PackageInitProc*)NULL);*/

  // find current working directory, and set as mount point
/*  {
    Tcl_DString pwd;
    Tcl_DStringInit(&pwd);
    Tcl_GetCwd(interp, &pwd);
    Zvfs_Mount(interp, (char *)Tcl_GetNameOfExecutable(), 
	       Tcl_DStringValue(&pwd));
    Tcl_DStringFree(&pwd);
  }*/

  // Initialize Tcl and Tk
  if (Tcl_Init(interp))
    return TCL_ERROR;

  // Tk
  if (Tk_Init(interp))
    return TCL_ERROR;
//  Tcl_StaticPackage(interp,"Tk", Tk_Init, Tk_SafeInit);

#ifdef __WIN32__
  if (console) {
    if (Tk_CreateConsoleWindow(interp) == TCL_ERROR)
      return TCL_ERROR;
    Tcl_Eval(interp, "console show");
  }
#endif

  // Blt
/*  if (Blt_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "BLT", Blt_Init, 
		     (Tcl_PackageInitProc*)NULL);*/

  //Tcl_PkgRequire(interp, "BLT", "2.4", 0);
  // Tktable
  /*if (Tktable_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "Tktable", Tktable_Init, 
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "Tktable", "2.9", 0);

  // Checkdns
  if (Checkdns_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "checkdns", Checkdns_Init, 
		     (Tcl_PackageInitProc*)NULL);

  // Saotk
  if (Saotk_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "saotk", Saotk_Init,
		     (Tcl_PackageInitProc*)NULL);

  // Tkhtml
  if (Tkhtml_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "tkhtml", Tkhtml_Init,
		     (Tcl_PackageInitProc*)NULL);

  // Tclxpa
/*  if (Tclxpa_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "Tclxpa", Tclxpa_Init,
		     (Tcl_PackageInitProc*)NULL);*/

  //Tcl_PkgRequire(interp, "Tclxpa", "1.0", 0);

  // IIS
  if (Iis_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "iis", Iis_Init, 
		     (Tcl_PackageInitProc*)NULL);

  // Tkmpeg
  if (Tkmpeg_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "tkmpeg", Tkmpeg_Init, 
		     (Tcl_PackageInitProc*)NULL);

  // Tkimg
  /*if (Tkimg_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "img", 
		     Tkimg_Init,(Tcl_PackageInitProc*)NULL);*/

  //Tcl_PkgRequire(interp, "img", "1.3", 0);

  // zlibtcl
/*  if (Zlibtcl_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "zlibtcl", Zlibtcl_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "zlibtcl", "1.2", 0);

  // jpegtcl
  /*if (Jpegtcl_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "jpegtcl", Jpegtcl_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "jpegtcl", "1.0", 0);

  // Tkimgjpeg
/*  if (Tkimgjpeg_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "jpeg", Tkimgjpeg_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "jpeg", "1.3", 0);

  // Tifftcl
/*  if (Tifftcl_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "tifftcl", Tifftcl_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "tifftcl", "3.8.2", 0);

  // Tkimgtiff
/*  if (Tkimgtiff_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "tiff", Tkimgtiff_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "tiff", "1.3", 0);

  // Pngtcl
/*  if (Pngtcl_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "pngtcl", Pngtcl_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "pngtcl", "1.2.12", 0);

  // Tkimgpng
/*  if (Tkimgpng_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "png", Tkimgpng_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "png", "1.3", 0);

  // Tkimggif
/*  if (Tkimggif_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "gif", Tkimggif_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "gif", "1.3", 0);

  // Tkimgppm
/*  if (Tkimgppm_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "ppm", Tkimgppm_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "ppm", "1.3", 0);
  // Tkimgbmp
/*  if (Tkimgbmp_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "bmp", Tkimgbmp_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "bpm", "1.3", 0);
  // Tkimgbmp

  // Tkimgxbm
/*  if (Tkimgxbm_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "xbm", Tkimgxbm_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "bpm", "1.3", 0);
  // Tkimgwindow
  /*if (Tkimgwindow_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "window", Tkimgwindow_Init,
		     (Tcl_PackageInitProc*)NULL);*/
  //Tcl_PkgRequire(interp, "window", "1.3", 0);

#ifdef _MACOSX
  if (Tkmacosx_Init(interp) == TCL_ERROR)
    return TCL_ERROR;
  Tcl_StaticPackage (interp, "macosx", Tkmacosx_Init,
		     (Tcl_PackageInitProc*)NULL);
#endif

  // Variables
/*  Tcl_SetVar(interp, "auto_path", "./zvfsmntpt/tcl8.4 ./zvfsmntpt/tk8.4 ./zvfsmntpt/blt2.4 ./zvfsmntpt/tcllib1.6 ./zvfsmntpt/src", TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "tcl_libPath", "./zvfsmntpt/tcl8.4", TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "blt_library", "./zvfsmntpt/blt2.4", TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "blt_libPath", "./zvfsmntpt/blt2.4", TCL_GLOBAL_ONLY);
  Tcl_SetVar2(interp, "env", "TK_TABLE_LIBRARY", "", TCL_GLOBAL_ONLY)
  Tcl_SetVar(interp, "blt_library", LIBDIR"/blt2.4", TCL_GLOBAL_ONLY);*/

  //Tcl_SetVar(interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);

  return TCL_OK;
}
